@@ -15,7 +15,7 @@ const int TeleWindow::YMargin = 15;
 
 
 TeleWindow::TeleWindow(Display *dpy)
-    :_dpy(dpy)
+    :_dpy(dpy), _mappings(dpy)
 {
     int scr = DefaultScreen(_dpy);
     _rootWindow = RootWindow(_dpy, scr);
@@ -23,6 +23,9 @@ TeleWindow::TeleWindow(Display *dpy)
     Visual *visual = DefaultVisual(_dpy, scr);
     Colormap colormap = DefaultColormap(_dpy, scr);
 
+    _width = DisplayWidth(_dpy, scr);
+    _height = DisplayHeight(_dpy, scr);
+
 
     // Initializing imlib2
 
@@ -32,162 +35,66 @@ TeleWindow::TeleWindow(Display *dpy)
 
 
     // Loading background
+    _bgPixmap = 0;
+    reloadBackground();
 
-    _bgPixmap = XCreatePixmap(_dpy, _rootWindow, 800, 480, depth);
-
-    printf("Loading background from '%s'\n", Settings::instance()->backgroundFilename());
-    printf("Background mode: %d\n", Settings::instance()->backgroundMode());
-    Imlib_Image background = imlib_load_image(Settings::instance()->backgroundFilename());
-    if (background == 0)
-        printf("Cannot load background\n");
-    else
-    {
-        Imlib_Image bgToDraw = 0;
-
-        int bgXpos = 0;
-        int bgYpos = 0;
-
-        imlib_context_set_image(background);
-        switch (Settings::instance()->backgroundMode())
-        {
-            case Settings::Stretched:
-            {
-                bgToDraw = imlib_create_cropped_scaled_image(
-                    0, 0,
-                    imlib_image_get_width(), imlib_image_get_height(),
-                    800, 480
-                );
-                break;
-            }
-
-            case Settings::Centered:
-            {
-                // Hildon's logic is not so easy in this case
-
-//                bgToDraw = imlib_create_cropped_image(
-//                    (imlib_image_get_width() - 800) / 2,
-//                    (imlib_image_get_height()- 480) / 2,
-//                    800, 480
-//                );
-
-                double scalex = 1.0;
-                double scaley = 1.0;
-
-                if (imlib_image_get_width() > 800 * 2)
-                    scalex = (double)imlib_image_get_width() / 2.0 / 800;
-                if (imlib_image_get_height() > 480 * 2)
-                    scaley = (double)imlib_image_get_height() / 2.0 / 480;
-
-                double scale = scalex;
-                if (scaley > scale)
-                    scale = scaley;
-
-                int newwidth = (int)(scale * 800);
-                int newheight= (int)(scale * 480);
-
-                bgToDraw = imlib_create_cropped_scaled_image(
-                    (imlib_image_get_width() - newwidth) / 2,
-                    (imlib_image_get_height()- newheight)/ 2,
-                    newwidth, newheight,
-                    800, 480
-                );
-
-                break;
-            }
-
-            case Settings::Scaled: case Settings::Cropped:
-            {
-                double scalex = 800.0 / imlib_image_get_width();
-                double scaley = 480.0 / imlib_image_get_height();
-                double scale = scalex;
-
-                if (Settings::instance()->backgroundMode() == Settings::Scaled)
-                {
-                    if (scaley < scale) scale = scaley;
-                }
-                else
-                {
-                    if (scaley > scale) scale = scaley;
-                }
-
-                int newwidth = (int)(imlib_image_get_width() * scale);
-                int newheight= (int)(imlib_image_get_height() * scale);
-
-                bgToDraw = imlib_create_cropped_scaled_image(
-                    0, 0,
-                    imlib_image_get_width(), imlib_image_get_height(),
-                    newwidth, newheight
-                );
-
-                bgXpos = (800 - newwidth) / 2;
-                bgYpos = (480 - newheight) / 2;
-                break;
-            }
-
-            default:
-                printf("Unknown background mode: %d\n", Settings::instance()->backgroundMode());
-        }
-
-        imlib_free_image();
-
-        imlib_context_set_image(bgToDraw);
-        imlib_context_set_drawable(_bgPixmap);
-
-        imlib_render_image_on_drawable(bgXpos, bgYpos);
-
-        imlib_free_image();
-    }
 
 
 // Loading header images
 
-    // We need RGBA visual for transparency
-    XVisualInfo rgbaVisual;
-    if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
-        fprintf(stderr, "Cannot find rgba visual\n");
-    XRenderPictFormat *rgbaFormat = XRenderFindVisualFormat(_dpy, rgbaVisual.visual);
+    bool ok = true;
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerLeftFilename(),
+        _rootWindow,
+        &_headerLeftPix,
+        &_headerLeftPict,
+        &_headerLeftWidth,
+        &_headerHeight);
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerRightFilename(),
+        _rootWindow,
+        &_headerRightPix,
+        &_headerRightPict,
+        &_headerRightWidth,
+        0);
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerMiddleFilename(),
+        _rootWindow,
+        &_headerMiddlePix,
+        &_headerMiddlePict,
+        0, 0);
 
-    // Blend == 0 makes imlib to copy alpha channel instead of using it for blending
-    imlib_context_set_visual(rgbaVisual.visual);
-    imlib_context_set_blend(0);
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerLeftSelectedFilename(),
+        _rootWindow,
+        &_headerLeftSelectedPix,
+        &_headerLeftSelectedPict,
+        0, 0);
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerRightSelectedFilename(),
+        _rootWindow,
+        &_headerRightSelectedPix,
+        &_headerRightSelectedPict,
+        0, 0);
+    ok = ok && loadARGBImage(
+        Settings::instance()->headerMiddleSelectedFilename(),
+        _rootWindow,
+        &_headerMiddleSelectedPix,
+        &_headerMiddleSelectedPict,
+        0, 0);
 
-    Imlib_Image headerLeft = imlib_load_image(Settings::instance()->headerLeftFilename());
-    Imlib_Image headerRight = imlib_load_image(Settings::instance()->headerRightFilename());
-    Imlib_Image headerMiddle = imlib_load_image(Settings::instance()->headerMiddleFilename());
-    if (headerLeft == 0 || headerRight == 0 || headerMiddle == 0)
-    {
-        fprintf(stderr, "Cannot load header pixmaps\n");
-        exit(1); // Ugly, i know
-    }
+    if (! ok)
+        printf("Cannot load all pixmaps\n");
 
-    imlib_context_set_image(headerLeft);
-    _headerLeftWidth = imlib_image_get_width();
-    _headerLeftPix = XCreatePixmap(_dpy, _rootWindow, imlib_image_get_width(), imlib_image_get_height(), 32);
-    imlib_context_set_drawable(_headerLeftPix);
-    imlib_render_image_on_drawable(0, 0);
-    imlib_free_image();
-    _headerLeftPict = XRenderCreatePicture(_dpy, _headerLeftPix, rgbaFormat, 0, 0);
+    XRenderPictureAttributes pictureAttrs;
+    pictureAttrs.repeat = RepeatNormal;
+    XRenderChangePicture(_dpy, _headerMiddlePict, CPRepeat, &pictureAttrs);
+    XRenderChangePicture(_dpy, _headerMiddleSelectedPict, CPRepeat, &pictureAttrs);
 
-    imlib_context_set_image(headerRight);
-    _headerRightWidth = imlib_image_get_width();
-    _headerRightPix = XCreatePixmap(_dpy, _rootWindow, imlib_image_get_width(), imlib_image_get_height(), 32);
-    imlib_context_set_drawable(_headerRightPix);
-    imlib_render_image_on_drawable(0, 0);
-    imlib_free_image();
-    _headerRightPict = XRenderCreatePicture(_dpy, _headerRightPix, rgbaFormat, 0, 0);
-
-    imlib_context_set_image(headerMiddle);
-    _headerHeight = imlib_image_get_height();
-    _headerMiddlePix = XCreatePixmap(_dpy, _rootWindow, imlib_image_get_width(), imlib_image_get_height(), 32);
-    imlib_context_set_drawable(_headerMiddlePix);
-    imlib_render_image_on_drawable(0, 0);
-    imlib_free_image();
-
-    XRenderPictureAttributes pictAttrs;
-    pictAttrs.repeat = RepeatNormal;
-    _headerMiddlePict = XRenderCreatePicture(_dpy, _headerMiddlePix, rgbaFormat, CPRepeat, &pictAttrs);
 
     XRenderParseColor(_dpy, Settings::instance()->borderColor(), &_borderColor);
+    XRenderParseColor(_dpy, Settings::instance()->borderActiveColor(), &_borderActiveColor);
+
 
     _borderWidth = Settings::instance()->borderWidth();
     _textLeftMargin = Settings::instance()->textLeftMargin();
@@ -212,12 +119,11 @@ TeleWindow::TeleWindow(Display *dpy)
     // Creating main window
 
     XSetWindowAttributes createAttrs;
-    createAttrs.background_pixmap = _bgPixmap;
-    createAttrs.save_under = True; // Thus seems to not work, but let's keep it
+    createAttrs.save_under = True; // This seems to not work, but let's keep it
 
-    _win = XCreateWindow(_dpy, _rootWindow, 0, 0, 800, 480,
+    _win = XCreateWindow(_dpy, _rootWindow, 0, 0, _width, _height,
         0, depth, InputOutput, visual,
-        CWSaveUnder,// 0,//CWBackPixmap,
+        CWSaveUnder,
         &createAttrs);
 
     _gc = XCreateGC(_dpy, _win, 0, 0);
@@ -232,9 +138,6 @@ TeleWindow::TeleWindow(Display *dpy)
         KeyReleaseMask
     );
 
-    _width = 800;
-    _height = 480;
-
 
     // Initializing Xft
 
@@ -249,29 +152,26 @@ TeleWindow::TeleWindow(Display *dpy)
 
 
     // Double buffering pixmap
-    _bufferPix = XCreatePixmap(_dpy, _rootWindow, _width, _height, depth);
+    _bufferPix = 0;
+    _xrenderFormat = 0;
+    recreateBufferPixmap();
+
+
+    XRenderPictureAttributes pa;
+    pa.subwindow_mode = IncludeInferiors;
+
+    _picture = XRenderCreatePicture(_dpy, _win, _xrenderFormat, CPSubwindowMode, &pa);
 
     // Xft Draw object
     _xftDraw = XftDrawCreate(_dpy, _bufferPix, visual, colormap);
 
+
     _shown = false;
 
 
 
-    // XRender picture for main window and double buffer
-    XWindowAttributes attrs;
-    XGetWindowAttributes(_dpy, _win, &attrs);
-    XRenderPictFormat *format = XRenderFindVisualFormat(_dpy, attrs.visual);
 
-    XRenderPictureAttributes pa;
-    pa.subwindow_mode = IncludeInferiors;
-
-    _picture = XRenderCreatePicture(_dpy, _win, format, CPSubwindowMode, &pa);
-    _bufferPicture = XRenderCreatePicture(_dpy, _bufferPix, format, 0, 0);
-
-
-    // For intercepting MapNotify, UnmapNotify and DestroyNotify
-    XSelectInput(_dpy, _rootWindow, SubstructureNotifyMask);
+    XSelectInput(_dpy, _rootWindow, StructureNotifyMask | PropertyChangeMask);
 
 
     // Hotkey keycode
@@ -279,8 +179,10 @@ TeleWindow::TeleWindow(Display *dpy)
     _hotKeyCode = XKeysymToKeycode(_dpy, keysym);
     _hotKeyPressed = false;
 
+    _activeThumbnail = 0;
 
-    updateThumbnailsList();
+
+    markThumbnailsListDirty();
 }
 
 TeleWindow::~TeleWindow()
@@ -289,6 +191,8 @@ TeleWindow::~TeleWindow()
         delete *i;
 
 
+    XftDrawDestroy(_xftDraw);
+
     XRenderFreePicture(_dpy, _picture);
     XRenderFreePicture(_dpy, _bufferPicture);
 
@@ -320,16 +224,19 @@ Window TeleWindow::window()
     return _win;
 }
 
-Picture TeleWindow::picture()
-{
-    return _picture;
-}
-
 
 void TeleWindow::show()
 {
     _shown = true;
 
+    Window activeWindow = XTools::activeWindow();
+
+    _activeThumbnail = 0;
+    for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
+        if ((*i)->clientWindow() == activeWindow)
+            _activeThumbnail = *i;
+
+
     XMapWindow(_dpy, _win);
 
     XEvent event;
@@ -347,6 +254,10 @@ void TeleWindow::show()
     XSendEvent(_dpy, _rootWindow, False, SubstructureNotifyMask, &event);
 
     XTools::switchToWindow(_win);
+
+
+//    if (_activeThumbnail)
+//        animate(_activeThumbnail, true);
 }
 
 void TeleWindow::hide()
@@ -363,8 +274,21 @@ bool TeleWindow::shown()
 
 
 
+void TeleWindow::markThumbnailsListDirty()
+{
+    _thumbnailsListDirty = true;
+
+    if (_shown)
+        updateThumbnailsList();
+}
+
+
+
 void TeleWindow::updateThumbnailsList()
 {
+    if (_thumbnailsListDirty == false)
+        return;
+
     bool wasChanged = false;
 
 
@@ -403,6 +327,8 @@ void TeleWindow::updateThumbnailsList()
 
     if (wasChanged)
         layoutThumbnails();
+
+    _thumbnailsListDirty = false;
 }
 
 
@@ -462,8 +388,8 @@ void TeleWindow::layoutThumbnails()
     else
     {
         // If there is only one window we don't want it become to big
-        thumbWidth = 800 * 2 / 3;
-        thumbHeight = 480 * 2 / 3;
+        thumbWidth = _width * 2 / 3;
+        thumbHeight = _height * 2 / 3;
     }
 
     int widthWithBorder = thumbWidth + 2*_borderWidth;
@@ -499,7 +425,8 @@ void TeleWindow::layoutThumbnails()
         }
     }
 
-    paint();
+    if (_shown)
+        paint();
 }
 
 
@@ -565,8 +492,28 @@ void TeleWindow::onRootEvent(XEvent *event)
         onHotKeyPress();
     if (event->type == KeyRelease && event->xkey.keycode == _hotKeyCode)
         onHotKeyRelease();
-    else if (event->type == MapNotify)
-        updateThumbnailsList();
+//    else if (event->type == MapNotify)
+//        updateThumbnailsList();
+    else if (event->type == PropertyNotify)
+    {
+//        char *name = XGetAtomName(_dpy, event->xproperty.atom);
+//        printf("Root PropertyNotify: %s\n", name);
+//        XFree(name);
+
+        if (_shown && event->xproperty.atom == XTools::_NET_ACTIVE_WINDOW)
+            if (XTools::activeWindow() != _win)
+                hide();
+
+        if (event->xproperty.atom == XTools::_NET_CLIENT_LIST)
+            markThumbnailsListDirty();
+    }
+    else if (event->type == ConfigureNotify)
+    {
+        if (event->xconfigure.width != _width || event->xconfigure.height != _height)
+        {
+            setNewSize(event->xconfigure.width, event->xconfigure.height);
+        }
+    }
 //    else
 //        printf("Unknown root event: %d\n", event->type);
 }
@@ -592,6 +539,10 @@ void TeleWindow::onEvent(XEvent *event)
         if (relayout)
             layoutThumbnails();
     }
+    else if (event->type == KeyPress)
+    {
+        _mappings.handleEvent(this, Mapping::Press, event->xkey.keycode);
+    }
 }
 
 
@@ -608,65 +559,67 @@ void TeleWindow::paint()
         0, 0, _width, _height, 0, 0
     );
 
-    XRenderColor borderColor;
-    borderColor.red     = 0x8000;
-    borderColor.green   = 0x8000;
-    borderColor.blue    = 0x8000;
-    borderColor.alpha   = 0x8000;
 
     for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
     {
-//        XRenderFillRectangle(_dpy, PictOpOver, _bufferPicture, &borderColor,
-//            _scrollX + (*i)->x() - BorderSize, _scrollY + (*i)->y() - TitleSize,
-//            (*i)->width() + 2*BorderSize, (*i)->height() + BorderSize + TitleSize
-//        );
-
         int x = (*i)->x();
         int y = (*i)->y();
         int w = (*i)->realWidth();
         int h = (*i)->realHeight();
+        int xoffset = ((*i)->width() - w) / 2;
+        int yoffset = ((*i)->height() - h) / 2;
+
+        bool selected = (*i) == _activeThumbnail;
+        XRenderColor borderColor = selected ? _borderActiveColor : _borderColor;
+
+        Picture left = selected ? _headerLeftSelectedPict : _headerLeftPict;
+        Picture right = selected ? _headerRightSelectedPict : _headerRightPict;
+        Picture middle = selected ? _headerMiddleSelectedPict : _headerMiddlePict;
 
         XRenderComposite(_dpy, PictOpOver,
-            _headerLeftPict, None, _bufferPicture,
+            left, None, _bufferPicture,
             0, 0,
             0, 0,
-            _scrollX + x - _borderWidth,
-            _scrollY + y - _headerHeight,
+            _scrollX + x + xoffset - _borderWidth,
+            _scrollY + y + yoffset - _headerHeight,
             _headerLeftWidth, _headerHeight
         );
 
         XRenderComposite(_dpy, PictOpOver,
-            _headerRightPict, None, _bufferPicture,
+            right, None, _bufferPicture,
             0, 0,
             0, 0,
-            _scrollX + x + w + _borderWidth - _headerRightWidth,
-            _scrollY + y - _headerHeight,
+            _scrollX + x + xoffset + w + _borderWidth - _headerRightWidth,
+            _scrollY + y + yoffset - _headerHeight,
             _headerRightWidth, _headerHeight
         );
 
         XRenderComposite(_dpy, PictOpSrc,
-            _headerMiddlePict, None, _bufferPicture,
+            middle, None, _bufferPicture,
             0, 0,
             0, 0,
-            _scrollX + x - _borderWidth + _headerLeftWidth,
-            _scrollY + y - _headerHeight,
+            _scrollX + x + xoffset - _borderWidth + _headerLeftWidth,
+            _scrollY + y + yoffset - _headerHeight,
             w + 2 * _borderWidth - _headerLeftWidth - _headerRightWidth,
             _headerHeight
         );
 
         // Left border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &_borderColor,
-            _scrollX + x - _borderWidth, _scrollY + y,
+        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
+            _scrollX + x + xoffset - _borderWidth,
+            _scrollY + y + yoffset,
             _borderWidth, h
         );
         // Right border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &_borderColor,
-            _scrollX + x + w, _scrollY + y,
+        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
+            _scrollX + x + xoffset + w,
+            _scrollY + y + yoffset,
             _borderWidth, h
         );
         // Bottom border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &_borderColor,
-            _scrollX + x - _borderWidth, _scrollY + y + h,
+        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
+            _scrollX + x + xoffset - _borderWidth,
+            _scrollY + y + yoffset + h,
             w + 2*_borderWidth, _borderWidth
         );
     }
@@ -682,10 +635,13 @@ void TeleWindow::paint()
 
     for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
     {
+        int xoffset = ((*i)->width() - (*i)->realWidth()) / 2;
+        int yoffset = ((*i)->height() - (*i)->realHeight()) / 2;
+
         XRectangle rect = {
-            _scrollX + (*i)->x() - _borderWidth + _textLeftMargin,
-            _scrollY + (*i)->y() - _headerHeight,
-            (*i)->width() + 2*_borderWidth - _textLeftMargin - _textRightMargin,
+            _scrollX + (*i)->x() + xoffset - _borderWidth + _textLeftMargin,
+            _scrollY + (*i)->y() + yoffset - _headerHeight,
+            (*i)->realWidth() + 2*_borderWidth - _textLeftMargin - _textRightMargin,
             _headerHeight
         };
         Region clip = XCreateRegion();
@@ -694,29 +650,17 @@ void TeleWindow::paint()
         XftDrawSetClip(_xftDraw, clip);
 
         XftDrawStringUtf8(_xftDraw, &fontColor, _xftFont, 
-            _scrollX + (*i)->x() - _borderWidth + _textLeftMargin,
-            _scrollY + (*i)->y() + Settings::instance()->textYOffset(),
+            _scrollX + (*i)->x() + xoffset - _borderWidth + _textLeftMargin,
+            _scrollY + (*i)->y() + yoffset + Settings::instance()->textYOffset(),
             (const FcChar8*)(*i)->title(),
             strlen((*i)->title())
         );
         
         XDestroyRegion(clip);
-
-
-
-//        XRenderComposite(_dpy, PictOpOver,
-//            _closePicture, None, _bufferPicture,
-//            0, 0,
-//            0, 0,
-//            _scrollX + (*i)->x() + (*i)->width() - _closeImageWidth,
-//            _scrollY + (*i)->y() - TitleSize + _closeImageYOffset,
-//            _closeImageWidth, _closeImageHeight
-//        );
-
     }
 
     for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
-        (*i)->paint(_bufferPicture);
+        (*i)->paint();
 
 
 
@@ -758,7 +702,10 @@ void TeleWindow::onHotKeyPress()
 
     _hotKeyPressed = true;
 
-    show();
+    if (! _shown)
+        show();
+    else
+        _mappings.handleEvent(this, Mapping::Press, _hotKeyCode);
 }
 
 void TeleWindow::onHotKeyRelease()
@@ -812,6 +759,7 @@ void TeleWindow::onButtonRelease(XButtonEvent *event)
 
             if (missed && (*i)->inside(- _scrollX + event->x, - _scrollY + event->y))
             {
+//                animate(*i, false);
                 (*i)->switchToClient();
                 hide();
                 missed = false;
@@ -819,7 +767,10 @@ void TeleWindow::onButtonRelease(XButtonEvent *event)
         }
 
         if (missed)
+        {
+            hide();
             XTools::showDesktop();
+        }
     }
 }
 
@@ -853,3 +804,395 @@ void TeleWindow::onIdle()
         _repaintOnIdle = false;
     }
 }
+
+
+void TeleWindow::internalCommand(const char *action)
+{
+    if (strcmp(action, "switchToSelected") == 0)
+    {
+        if (_activeThumbnail != 0)
+            _activeThumbnail->switchToClient();
+        else
+            XTools::showDesktop();
+        hide();
+    }
+    else if (strcmp(action, "selectNext") == 0)
+    {
+        if (_activeThumbnail == 0)
+        {
+            if (_thumbnails.size() > 0)
+                _activeThumbnail = *_thumbnails.head();
+        }
+        else
+        {
+            int index = 0;
+            bool changed = false;
+            for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
+                if ((*i) == _activeThumbnail)
+                {
+                    if (index < _thumbnails.size() - 1)
+                        _activeThumbnail = *(++i);
+                    else
+                        _activeThumbnail = *_thumbnails.head();
+                    changed = true;
+                    break;
+                }
+
+            if (! changed)
+                _activeThumbnail = *_thumbnails.head();
+        }
+
+        paint();
+    }
+    else if (strcmp(action, "selectPrev") == 0)
+    {
+        if (_activeThumbnail == 0)
+        {
+            if (_thumbnails.size() > 0)
+                _activeThumbnail = *_thumbnails.tail();
+        }
+        else
+        {
+            int index = 0;
+            bool changed = false;
+            for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
+                if ((*i) == _activeThumbnail)
+                {
+                    if (index > 0)
+                        _activeThumbnail = *(--i);
+                    else
+                        _activeThumbnail = *_thumbnails.tail();
+                    changed = true;
+                    break;
+                }
+
+            if (! changed)
+                _activeThumbnail = *_thumbnails.tail();
+        }
+
+        paint();
+    }
+    else
+    {
+        fprintf(stderr, "Unknown internal command: %s\n", action);
+    }
+}
+
+
+
+void TeleWindow::setNewSize(int width, int height)
+{
+    printf("setNewSize(%d, %d)\n", width, height);
+
+    _width = width;
+    _height = height;
+
+    XResizeWindow(_dpy, _win, _width, _height);
+
+    recreateBufferPixmap();
+
+    reloadBackground();
+
+    layoutThumbnails();
+}
+
+
+
+void TeleWindow::reloadBackground()
+{
+    if (_bgPixmap != 0)
+        XFreePixmap(_dpy, _bgPixmap);
+
+    _bgPixmap = XCreatePixmap(_dpy, _rootWindow,
+        _width, _height,
+        DefaultDepth(_dpy, DefaultScreen(_dpy)));
+
+    printf("Loading background from '%s'\n", Settings::instance()->backgroundFilename());
+    printf("Background mode: %d\n", Settings::instance()->backgroundMode());
+    Imlib_Image background = imlib_load_image(Settings::instance()->backgroundFilename());
+    if (background == 0)
+        printf("Cannot load background\n");
+    else
+    {
+        Imlib_Image bgToDraw = 0;
+
+        int bgXpos = 0;
+        int bgYpos = 0;
+
+        imlib_context_set_image(background);
+        switch (Settings::instance()->backgroundMode())
+        {
+            case Settings::Stretched:
+            {
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    0, 0,
+                    imlib_image_get_width(), imlib_image_get_height(),
+                    _width, _height
+                );
+                break;
+            }
+
+            case Settings::Centered:
+            {
+                // Hildon's logic is not so easy in this case
+
+//                bgToDraw = imlib_create_cropped_image(
+//                    (imlib_image_get_width() - _width) / 2,
+//                    (imlib_image_get_height()- _height) / 2,
+//                    _width, _height
+//                );
+
+                double scalex = 1.0;
+                double scaley = 1.0;
+
+                if (imlib_image_get_width() > _width * 2)
+                    scalex = (double)imlib_image_get_width() / 2.0 / _width;
+                if (imlib_image_get_height() > _height * 2)
+                    scaley = (double)imlib_image_get_height() / 2.0 / _height;
+
+                double scale = scalex;
+                if (scaley > scale)
+                    scale = scaley;
+
+                int newwidth = (int)(scale * _width);
+                int newheight= (int)(scale * _height);
+
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    (imlib_image_get_width() - newwidth) / 2,
+                    (imlib_image_get_height()- newheight)/ 2,
+                    newwidth, newheight,
+                    _width, _height
+                );
+
+                break;
+            }
+
+            case Settings::Scaled: case Settings::Cropped:
+            {
+                double scalex = (double)_width / imlib_image_get_width();
+                double scaley = (double)_height / imlib_image_get_height();
+                double scale = scalex;
+
+                if (Settings::instance()->backgroundMode() == Settings::Scaled)
+                {
+                    if (scaley < scale) scale = scaley;
+                }
+                else
+                {
+                    if (scaley > scale) scale = scaley;
+                }
+
+                int newwidth = (int)(imlib_image_get_width() * scale);
+                int newheight= (int)(imlib_image_get_height() * scale);
+
+                bgToDraw = imlib_create_cropped_scaled_image(
+                    0, 0,
+                    imlib_image_get_width(), imlib_image_get_height(),
+                    newwidth, newheight
+                );
+
+                bgXpos = (_width - newwidth) / 2;
+                bgYpos = (_height - newheight) / 2;
+                break;
+            }
+
+            default:
+                printf("Unknown background mode: %d\n", Settings::instance()->backgroundMode());
+        }
+
+        imlib_free_image();
+
+        imlib_context_set_image(bgToDraw);
+        imlib_context_set_drawable(_bgPixmap);
+        imlib_context_set_visual(DefaultVisual(_dpy, DefaultScreen(_dpy)));
+
+
+        imlib_render_image_on_drawable(bgXpos, bgYpos);
+
+        imlib_free_image();
+    }
+}
+
+
+void TeleWindow::recreateBufferPixmap()
+{
+    if (_bufferPix)
+    {
+        XRenderFreePicture(_dpy, _bufferPicture);
+        XFreePixmap(_dpy, _bufferPix);
+    }
+
+    int scr = DefaultScreen(_dpy);
+    int depth = DefaultDepth(_dpy, scr);
+    Visual *visual = DefaultVisual(_dpy, scr);
+
+    if (_xrenderFormat == 0)
+    {
+        _xrenderFormat = XRenderFindVisualFormat(_dpy, visual);
+        XVisualInfo rgbaVisual;
+        if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
+            fprintf(stderr, "Cannot find rgba visual\n");
+        _xrenderRGBAFormat = XRenderFindVisualFormat(_dpy, rgbaVisual.visual);
+    }
+
+    // Double-buffering pixmap
+    _bufferPix = XCreatePixmap(_dpy, _rootWindow, _width, _height, depth);
+
+    // XRender picture
+    _bufferPicture = XRenderCreatePicture(_dpy, _bufferPix, _xrenderFormat, 0, 0);
+}
+
+
+bool TeleWindow::loadARGBImage(
+    const char *filename,
+    Drawable parent,
+    Pixmap *pixmap,
+    Picture *picture,
+    int *width,
+    int *height)
+{
+    static Visual *defaultVisual = 0;
+    static XVisualInfo rgbaVisual;
+    static XRenderPictFormat *defaultFormat;
+    static XRenderPictFormat *rgbaFormat;
+    static Colormap colormap;
+
+    *pixmap = 0;
+    *picture = 0;
+
+    if (defaultVisual == 0)
+    {
+        // Initializing
+
+        int scr = DefaultScreen(_dpy);
+        defaultVisual = DefaultVisual(_dpy, scr);
+        colormap = DefaultColormap(_dpy, scr);
+        defaultFormat = XRenderFindVisualFormat(_dpy, defaultVisual);
+
+        if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
+        {
+            fprintf(stderr, "Cannot find rgba visual\n");
+            return false;
+        }
+        rgbaFormat = XRenderFindVisualFormat(_dpy, rgbaVisual.visual);
+    }
+
+
+    Imlib_Image image = imlib_load_image(filename);
+    if (image == 0)
+        return false;
+
+    imlib_context_set_image(image);
+    int w = imlib_image_get_width();
+    int h = imlib_image_get_height();
+
+    if (width) *width = w;
+    if (height) *height = h;
+
+    // Making image with premultiplied alpha
+    Imlib_Image premul = imlib_create_image(w, h);
+    imlib_context_set_image(premul);
+    imlib_context_set_color(0, 0, 0, 255);
+    imlib_image_fill_rectangle(0, 0, w, h);
+    imlib_context_set_blend(1);
+    imlib_blend_image_onto_image(image, 0, 0, 0, w, h, 0, 0, w, h);
+    imlib_image_copy_alpha_to_image(image, 0, 0);
+
+    *pixmap = XCreatePixmap(_dpy, parent, w, h, 32);
+    imlib_context_set_display(_dpy);
+    imlib_context_set_colormap(colormap);
+    imlib_context_set_visual(rgbaVisual.visual);
+    imlib_context_set_drawable(*pixmap);
+    imlib_context_set_blend(0);
+    imlib_render_image_on_drawable(0, 0);
+
+    imlib_free_image();
+    imlib_context_set_image(image);
+    imlib_free_image();
+
+    *picture = XRenderCreatePicture(_dpy, *pixmap, rgbaFormat, 0, 0);
+
+    return true;
+}
+
+
+void TeleWindow::animate(Thumbnail *thumb, bool toSmall)
+{
+        XFlush(_dpy);
+
+        
+        XWindowAttributes attrs;
+        XGetWindowAttributes(_dpy, thumb->clientWindow(), &attrs);
+
+        int bigx, bigy;
+        int bigw = attrs.width;
+        int bigh = attrs.height;
+        Window child;
+        XTranslateCoordinates(_dpy, thumb->clientWindow(), _rootWindow,
+            0, 0, &bigx, &bigy, &child);
+
+        int xoffset = (thumb->width() - thumb->realWidth()) / 2;
+        int yoffset = (thumb->height() - thumb->realHeight()) / 2;
+        int smallx = thumb->x() + xoffset;
+        int smally = thumb->y() + yoffset;
+        int smallw = thumb->realWidth();
+        int smallh = thumb->realHeight();
+
+
+        if (! toSmall)
+        {
+            int t;
+            t = smallx; smallx = bigx; bigx = t;
+            t = smally; smally = bigy; bigy = t;
+            t = smallw; smallw = bigw; bigw = t;
+            t = smallh; smallh = bigh; bigh = t;
+        }
+
+        int prevx = 0, prevy = 0, prevw = _width, prevh = _height;
+
+
+        double step = toSmall ? 0.33 : 0.50;
+        for (double a = 0.0; a <= 1.0; a += step)
+        {
+            int x = (int)(((double)bigx) * (1.0 - a) + ((double)smallx) * a);
+            int y = (int)(((double)bigy) * (1.0 - a) + ((double)smally) * a);
+            int w = (int)(((double)bigw) * (1.0 - a) + ((double)smallw) * a);
+            int h = (int)(((double)bigh) * (1.0 - a) + ((double)smallh) * a);
+
+            double scale = w;
+            scale /= attrs.width;
+
+            XTransform xform = {{
+                { XDoubleToFixed(1), XDoubleToFixed(0), XDoubleToFixed(0) },
+                { XDoubleToFixed(0), XDoubleToFixed(1), XDoubleToFixed(0) },
+                { XDoubleToFixed(0), XDoubleToFixed(0), XDoubleToFixed(scale) },
+            }};
+
+            XRenderSetPictureTransform(_dpy, thumb->clientPicture(), &xform);
+
+            if (toSmall)
+                XCopyArea(_dpy, _bufferPix, _win, _gc,
+                    prevx, prevy, prevw, prevh, prevx, prevy);
+
+            XRenderComposite(_dpy, PictOpSrc,
+                thumb->clientPicture(), None, _picture,
+                (int)(- (1.0 - scale) * thumb->clientDecoX()),
+                (int)(- (1.0 - scale) * thumb->clientDecoY()),
+                0, 0,
+                x, y,
+                w, h
+            );
+
+            prevx = x;
+            prevy = y;
+            prevw = w;
+            prevh = h;
+
+            XSync(_dpy, 0);
+
+            //struct timeval ts;
+            //ts.tv_sec = 0;
+            //ts.tv_usec = 1000;
+            //select(0, 0, 0, 0, &ts);
+        }
+}
