@@ -130,7 +130,7 @@ void Thumbnail::onClientEvent(XEvent *event)
         _cacheValid = false;
         if (_teleWindow->shown())
         {
-            paint(_teleWindow->picture());
+            paint();
             _teleWindow->onThumbRedrawed(this);
         }
 
@@ -143,12 +143,12 @@ void Thumbnail::onClientEvent(XEvent *event)
     else if (event->type == DestroyNotify)
     {
         _clientDestroyed = true;
-        _teleWindow->updateThumbnailsList();
-    }
-    else if (event->type == UnmapNotify)
-    {
-        _teleWindow->updateThumbnailsList();
+//        _teleWindow->updateThumbnailsList();
     }
+//    else if (event->type == UnmapNotify)
+//    {
+//        _teleWindow->updateThumbnailsList();
+//    }
     else if (event->type == PropertyNotify)
     {
         if (event->xproperty.atom == XTools::_NET_WM_NAME)
@@ -258,7 +258,7 @@ void Thumbnail::onResize()
     _cacheValid = false;
 
 
-    paint(_teleWindow->picture());
+    paint();
 }
 
 
@@ -269,7 +269,7 @@ void Thumbnail::onClientResize(XEvent *event)
 }
 
 
-void Thumbnail::paint(Picture target)
+void Thumbnail::paint()
 {
 //    XClearArea(_dpy, _teleWindow->window(), _x, _y, _width, _height, False);
 
@@ -286,11 +286,15 @@ void Thumbnail::paint(Picture target)
         _cacheValid = true;
     }
 
+    // FIXME: duplicating calculation from TeleWindow::paint()
+    int xoffset = (_width - _clientScaledWidth) / 2;
+    int yoffset = (_height - _clientScaledHeight) / 2;
+
     XCopyArea(_dpy, _cachePixmap, _teleWindow->_bufferPix, _teleWindow->_gc,
         0, 0,
         _clientScaledWidth, _clientScaledHeight,
-        _teleWindow->scrollX() + _x,// + _clientOffsetX,
-        _teleWindow->scrollY() + _y// + _clientOffsetY
+        _teleWindow->scrollX() + _x + xoffset,// + _clientOffsetX,
+        _teleWindow->scrollY() + _y + yoffset// + _clientOffsetY
     );
 }
 
