@@ -66,11 +66,16 @@ class Thumbnail
         int realWidth() { return _clientScaledWidth; }
         int realHeight() { return _clientScaledHeight; }
 
+        Picture clientPicture() { return _clientPict; }
+
+        int clientDecoX() { return _clientDecoX; }
+        int clientDecoY() { return _clientDecoY; }
+
         bool inside(int x, int y);
 
         void onClientEvent(XEvent *event);
 
-        void paint(Picture target);
+        void paint();
 
         void switchToClient();
         void closeClient();
